#
# Establish platform we're on
#
echo postremove running

if [ -f /etc/.relid ]
then
	HOST_OS=ncr
else
	HOST_OS=`uname -s`
fi

#
# Setup basic variables that vary by platform 
#
if [ "$HOST_OS" = "Windows" ]
then
	TONULL="NUL:"
else
	TONULL="/dev/null"
fi

#
# Establish PKG_BASE, including making sure directories are free of trailing
# slahses
#
if [ "Windows" != "$HOST_OS" ]
then
	PKG_BASE=`head -n 1 /etc/.snapvault`
	if [ "SunOS" = "$HOST_OS" ]
	then
		DB_DIR=`/usr/xpg4/bin/tail -n 1 /etc/.snapvault`
	else
		DB_DIR=`tail -n 1 /etc/.snapvault`
	fi
fi
STRIPPED_PKG_BASE="`echo $PKG_BASE | sed -e 's+\\$++' -e 's+/$++'`"
PKG_BASE="$STRIPPED_PKG_BASE"

#
# Establish the users temp directory, defaulting to /tmp
#
if [ "" = "$TMPDIR" ]
then
	echo "Using default /tmp as the temporary directory"
	TMPDIR=/tmp
	if [ "Windows" = "$HOST_OS" ]
	then
		TMPDIR="${TEMP}"
	fi
else
	echo "Using user specified temp directory $TMPDIR"
fi

if [ -z "$PKG_BASE" ]
then
	echo Cannot perform postremove - cannot establish PKG_BASE
	exit 1
fi

#
# Remove libraries from os directories
#
# UNIX    - Remove sym links
# WINDOWS - Delete copies of libraries
#
# NOTE: If you make change here, please also adjust the library install logic
#       in postinstall.sh
#
echo Removing libraries
for library in `cat "$TMPDIR"/liblist.txt`
do
	echo "$library" | grep libsv
	if [ "0" = $? ]
	then
		#
		# File is a library
		#
		if [ "Windows" = "$HOST_OS" ]
		then
			rm "${windir}/system32/${library}"
		elif [ "IRIX64" = "$HOST_OS" -o "IRIX" = "$HOST_OS" ]
		then
			if [ -h /usr/lib32/${library} ]
			then
				rm -f /usr/lib32/${library}
			fi
		else
			if [ -h /usr/lib/${library} ]
			then
				rm -f /usr/lib/${library}
			fi
		fi
	fi
done

rm -f "$TMPDIR"/liblist.txt

#
# Clean symbolic links to binaries
#
if [ "Windows" != "$HOST_OS" ]
then
	echo "Removing symbolic links"
	rm -f /usr/bin/svpmgr
	rm -f /usr/bin/svconfigurator
fi

rm -f /etc/.snapvault 2>${TONULL}

#
# Clean up Non-Native components.
#
if [ "SunOS" = "$HOST_OS" -o "Windows" = "$HOST_OS" ]
then
	echo "Removing $PKG_BASE"
	rm -rf "$PKG_BASE"
	if [ -d "$DB_DIR" ]
	then 
		echo "Removing $DB_DIR"
		rm -rf "$DB_DIR"
	fi
fi

exit 0

