#!/bin/sh

echo preinstall running

#
# Read current state of the parameter file. This allows us to see the
# user's chosen options, either as a result of running the request script 
# or from a response file.
#
if [ "$HOST_OS" = "Windows" ]
then
		TONULL="NUL:"
else
		TONULL="/dev/null"
fi

#
# Stop OSSV if it's running
#
if [ "$NV_UPGRADE" = TRUE ]
then
		ps "$SYSTEM_PS_ARGS" | grep -v 'grep' | grep 'svpmgr' > ${TONULL}
		ans=$?
		if [ $ans = 0 ]
    then
				echo "Shutting down OSSV services..."
				"$PKG_BASE"/bin/svpmgr shutdown > ${TONULL} 2>&1
				
				# Shutdown can take some time
				sleep 4

				# Need to remove links established by old svpmgr
				"$PKG_BASE"/bin/svpmgr remove > ${TONULL} 2>&1

	
				ps "$SYSTEM_PS_ARGS" | grep -v 'grep' | grep 'svpmgr'
				ans=$?
				if [ $ans = 0 ]
				then
						echo "Running processes are..."
						ps  "$SYSTEM_PS_ARGS"
						echo ""
						echo 'Failed to stop OSSV services. Please try again.'
						exit 1
				fi
    fi
fi

#
# Check that certain well known sv processes aren't running.
#
if [ "$NV_UPGRADE" = TRUE ]
then
		for proc in svconfigurator svpmgr svcmgr
		do
			echo Checking if $proc is running
			ps  "$SYSTEM_PS_ARGS" | grep -v 'grep' | grep -i $proc
			ans=$?
			if [ $ans = 0 ]
			then
					echo ""
					echo "The following NetVault process is currently running: $proc"
					echo "Please stop the process and try again"
					echo ""
					exit 1
			fi
		done
fi

#
# Check none of the OSSV libraries are in use if we're running on Windows.
# Even if the services shut down there can be problems if, for example, a
# database hook library has nv libs open
#
# Because move operations may suceed, even if the dll is in use, the
# libraries are copied and then an attempt is made to delete them. The
# delete will fail if the dll is in use.
#
# OSSV note: Although we shouldn't be interfacing with hook libraries this
# is probably still a worthwhile check
#
if [ "TRUE" = "$NV_UPGRADE" -a "Windows" = "$HOST_OS" ]
then
		if [ -d "$PKG_BASE"/lib/installcopy ]
		then
				rm "$PKG_BASE"/lib/installcopy/*
		else
				mkdir "$PKG_BASE"/lib/installcopy
		fi

		for lib in `ls "$PKG_BASE"/lib | grep libsv`
		do
			if [ -f "${windir}/system32"/"$lib" ]
			then
					cp "${windir}/system32"/"$lib" "$PKG_BASE"/lib/installcopy
					rm "${windir}/system32"/"$lib" > $TONULL
					if [ "0" != "$?" ]
					then
#
# make an exception for libsvwinevt.dll,
# since it has not changed from the previous release.
#
							if [ "libsvwinevt.dll" != "$lib" ]
							then
									echo Library "$lib" is in use
									LIBS_IN_USE=TRUE
							fi
					fi
			fi
		done

		cp "$PKG_BASE"/lib/installcopy/* "${windir}/system32" > $TONULL
		rm "$PKG_BASE"/lib/installcopy/* > $TONULL

		if [ "TRUE" = "$LIBS_IN_USE" ]
		then
				echo Some of the OSSV libraries are in use. OSSV cannot be upgraded
				echo until the programs using these libraries have been halted.
				for lib in `ls "$PKG_BASE"/lib | grep libsv`
				do
					ALLLIBS="$ALLLIBS $lib"
				done 
				nvdlllist $ALLLIBS
				exit 1
		else
				echo No OSSV libraries are currently in use
		fi
fi


# For AIX run slibclean to flush cached DLLs
if [ "$NV_UPGRADE" = TRUE -a "AIX" = "$HOST_OS" ]
then
  slibclean
fi

#
# If we're upgrading, take a copy of the config directory
#
if [ "$NV_UPGRADE" = TRUE ]
then
    if [ -d "$PKG_BASE"/config ]
    then
				if [ ! -d "$PKG_BASE"/config.pre ]
				then
						mkdir "$PKG_BASE"/config.pre
				fi
				
				echo "Saving original configuration information..."
				cp "$PKG_BASE"/config/* "$PKG_BASE"/config.pre
				if [ "0" != $? ]
				then
						echo "Failed to copy existing configuration files"
						exit 1
				fi
    fi
fi

if [ "$NV_UPGRADE" = TRUE ]
then
		#
		# If we're upgrading from 7.1 or later, it's an MSI to MSI upgrade.
		# We need to neutralize the preremove and postremove scripts so that
		# they dont damage the installation when features of the old install are
		# uninstalled.
		#
		# NOTE: May not be able to do version check here due to some early
		#       Corkers builds reporting their version as 2.0 rather than 7.1
		#
		if [ "$HOST_OS" = "Windows" ]
		then
				echo Nullifying preremove and postremove for msi to msi upgrade
				rm "$PKG_BASE"/installfiles/preremove.sh
				rm "$PKG_BASE"/installfiles/postremove.sh
				
				echo "echo nullified preremove" > "$PKG_BASE"/installfiles/preremove.sh
				echo "echo nullified postremove" > "$PKG_BASE"/installfiles/postremove.sh
		fi
fi

#
# If we are NOT doing an upgrade, and the $PKG_BASE or database directories
# already exist, we should remove them
#
if [ "$NV_UPGRADE" != "TRUE" ]
then
		if [ -d "$PKG_BASE" ]
		then
				echo "Removing existing '$PKG_BASE' directory"
				rm -rf "$PKG_BASE"
				if [ "0" != $? ]
				then
						echo "Failed to remove existing directory"
						exit 1
				fi
		fi

		if [ -d "$DB_DIR" ]
		then
				echo "Removing existing '$DB_DIR' directory"
				rm -rf "$DB_DIR"
				if [ "0" != $? ]
				then
						echo "Failed to remove existing database directory"
						exit 1
				fi
		fi
fi

exit 0
