#!/bin/sh
#
# This script installs Caffeine on Linux and copies the plugin libraries
# over.
#

STANDARDIFS=$IFS
IFS="
"            # By default, only break strings on a new line

#
# get_string:
#  Shell function to prompt the user for input
#
# Arguemnts:
#  $1 - Question to display to the user
#  $2 - Default answer if the user just presses return
#  $3 - Optional list of acceptable user responses. If provided, the user
#       will be continually prompted for an answer until one is provided
#       that matches one of these responses
#
get_string()
{
    GOT_THE_STRING=FALSE
    while [ "FALSE" = "$GOT_THE_STRING" ]
    do
      printf "%s " "$1"
      if [ -n "$3" ]
			then
					printf "(%s) " "$3"
      fi
      if [ -n "$2" ]
			then
					printf "[%s] " "$2"
      fi
      printf ": "
      read INPUT
      
      #
      # Assign to INPUT to default value if user just pressed return
      #
      if [ -z "$INPUT" ]
      then
					INPUT=$2
      fi
      
      #
      # Check if the input was acceptable
      #
      if [ -n "$3" ]
      then
					OLDIFS=$IFS
					IFS=$STANDARDIFS          # Break strings on spaces
					for allowed in $3
					do
						if [ "$allowed" = "$INPUT" ]
						then
								GOT_THE_STRING=TRUE
								break
						fi
					done
					IFS=$OLDIFS
      else
					#
					# All inputs were OK
					#
					GOT_THE_STRING=TRUE
      fi
      
      if [ "FALSE" = "$GOT_THE_STRING" ]
      then
					echo "'$INPUT' is not a valid answer. Please choose from '$3'"
      fi
    done
		
    GET_STRING_RESULT=$INPUT
		
    return 1
}


PKG_DIR=`head -n 1 /etc/.snapvault`
if [ ! -n "$PKG_DIR" ]
then
	echo "OSSV is not installed on this system"
	exit 1
fi

. "$PKG_DIR"/uninstall/commonfuncs.sh


#
# First, get the caffeine specific information
#
get_caffeine_info "$PKG_DIR"

#
# Check for incompatible version of caffeine.
#
if [ "Yes" = "$INCOMP_CAFFEINE_PRESENT" ]
then
	echo "You have an incompatible Host Agent $INCOMP_CAFFEINE_BUILD installed on this system."
	echo "$REQ_CAFFEINE_NAME cannot be installed on this system."
	exit 1
fi

if [ "No" = "$CAFFEINE_PRESENT" -o "Yes" = "$CAFFEINE_NEEDS_UPGRADE" ]
then
	check_caffeine_support
	if [ "No" = "$CAFFEINE_SUPPORTED_PLATFORM" ]
	then
		echo "$REQ_CAFFEINE_NAME is not supported on this platform"
		exit 1
	fi

	SHOULD_INSTALL_CAFFEINE="Yes"
	if [ "Yes" = "$CAFFEINE_NEEDS_UPGRADE" ]
	then
		get_string "Do you want to upgrade the existing installation of $REQ_CAFFEINE_NAME" "n" "y n"
		if [ "n" = "$GET_STRING_RESULT" ] || [ "N" = "$GET_STRING_RESULT" ]
		then
			SHOULD_INSTALL_CAFFEINE="No"
		fi
	fi

	if [ "Yes" = "$SHOULD_INSTALL_CAFFEINE" ]
	then
		"$PKG_DIR"/util/svsetstanza config programs.cfg null Caffeine OSSVInstalledCaffeine TRUE FALSE
		"$PKG_DIR"/manageability/agentsetup-linux.bin

		if [ 0 != "$?" ]
		then
			echo "Failed to install $REQ_CAFFEINE_NAME"
			exit 1
		fi
	fi
fi

echo "$REQ_CAFFEINE_NAME is installed at /opt/$REQ_CAFFEINE_BUILD"

echo "Stopping the $REQ_CAFFEINE_BUILD daemon"
"$REQ_CAFFEINE_FILE" stop

cp -f "$PKG_DIR"/manageability/*.so /opt/"$REQ_CAFFEINE_BUILD"/

echo "Starting the $REQ_CAFFEINE_BUILD daemon"
"$REQ_CAFFEINE_FILE" start

