#!/usr/bin/perl
# If you see an error message like:
# ./ossvinfo.pl: Command not found.
# Change the first line to point to correct perl binary.

# $Id: //depot/prod/ossv/R2.6x/tools/ossvinfo/Unix/ossvinfo.pl#1 $


use v5.6.1; # only runs with perl 5.6.1 and higher.
use Getopt::Std;
use English;
use File::Copy;
use File::Spec;

use strict;
use constant VERSION => '2.0';

my %options;
my ($user, $secondary) = ('root', undef);

getopts('s:l:v:a', \%options);

if(defined $options{v}) {
	print "Version: " . VERSION . "\n";
	exit(0);
}


my $allqtrees= 0;

defined $options{l} and $user = $options{l};
defined $options{s} and $secondary = $options{s};
defined $options{a} and $allqtrees= 1;

my $outdir = shift;
my $outfile = $outdir."/outfile";

unless($^O eq "solaris" or $^O eq "linux" or $^O eq "aix" or $^O eq "irix") {
	die "Unsupported platform $^O";
}

unless($outdir) {
	print "usage:\n\t$0 [ -l username:password ] [ -s secondary ] [-all] outdir\n";
	print "\t$0 -version\n";
	exit(1);
}

unless($EUID == 0) {
	die "Only root can run this program";
}

mkdir $outdir or opendir(DIR,$outdir) or 
	warn "Failed to create output folder $outdir $! \n";closedir DIR;
open(FD, "> $outfile") or die "Failed to open $outfile for writing: $!";


begin_section(\*FD, "REGISTRY");
my $installdir=print_registry(\*FD);
end_section(\*FD);

my $esxBinary= check_esx_binary();

if ($allqtrees eq 1) {
copy_all_changelog_files($outdir);
copy_all_trace_files($outdir);
}

begin_section(\*FD, "HOSTNAME");
print_hostname(\*FD);
end_section(\*FD);

begin_section(\*FD, "OS_VERSION");
print_os(\*FD);
end_section(\*FD);

begin_section(\*FD, "MEMORY");
print_memory(\*FD);
end_section(\*FD);

begin_section(\*FD, "VOLUME");
print_volume(\*FD);
end_section(\*FD);

begin_section(\*FD, "PROCESSES");
print_procs(\*FD);
end_section(\*FD);

begin_section(\*FD, "SNAPVAULT_STATUS");
print_snapvault_status(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "SNAPVAULT_DESTINATIONS");
print_snapvault_destinations(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "SNAPVAULT_STATUS_L");
print_snapvault_status_l(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "RELEASEDEF");
print_releasedef(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "CONFIGURE_CFG");
print_configure_cfg(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "SNAPVAULT");
print_snapvault(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "PATH_EXCLUDE");
print_path_exclude(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "FILE_EXCLUDE");
print_file_exclude(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "DB_DIR");
print_db_dir(\*FD, $installdir);
end_section(\*FD);

begin_section(\*FD, "SVINSTALLCHECK");
print_svinstallcheck(\*FD, $installdir);
end_section(\*FD);

if($secondary) {
	begin_section(\*FD, "SECONDARY_SYSCONFIG");
	print_rcmd(\*FD, $secondary, $user, "sysconfig -v");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_HOSTNAME");
	print_rcmd(\*FD, $secondary, $user, "hostname");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_VERSION");
	print_rcmd(\*FD, $secondary, $user, "version");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_SNAPMIRROR");
	print_rcmd(\*FD, $secondary, $user, "rdfile /etc/log/snapmirror");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_SNAPMIRROR_0");
	print_rcmd(\*FD, $secondary, $user, "rdfile /etc/log/snapmirror.0");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_SNAPVAULT_STATUS_L");
	print_rcmd(\*FD, $secondary, $user, "snapvault status -l");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_SNAPVAULT_STATUS_C");
	print_rcmd(\*FD, $secondary, $user, "snapvault status -c");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_QTREE_STATUS");
	print_rcmd(\*FD, $secondary, $user, "qtree status");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_SNAPVAULT_SNAP_SCHED");
	print_rcmd(\*FD, $secondary, $user, "snapvault snap sched");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_SNAP_LIST");
	print_rcmd(\*FD, $secondary, $user, "snap list");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_CIFS_SHARES");
	print_rcmd(\*FD, $secondary, $user, "cifs shares");
	end_section(\*FD);

	begin_section(\*FD, "SECONDARY_MESSAGES");
	print_rcmd(\*FD, $secondary, $user, "rdfile /etc/messages");
	end_section(\*FD);

}

if ($esxBinary)
{
	begin_section(\*FD, "DIR LISTING OF VMFS/VOLUMES");
	print_esx_volumes(\*FD);
	end_section(\*FD);

	begin_section(\*FD, "OSSV_VMWARE.CFG FILE");
	print_vmware_cfg(\*FD, $installdir);
	end_section(\*FD);

	begin_section(\*FD, "VMWARE VMX FILES");
	copy_vmware_vmxfiles(\*FD, $outdir);
	end_section(\*FD);

	begin_section(\*FD, "VMINFO");
	get_vmware_vminfo(\*FD, $installdir);
	end_section(\*FD);
}

begin_section(\*FD, "DATE");
print FD `date`;
end_section(\*FD);


close FD;

# ##############################

sub begin_section($$) {
	my($D, $sec) = @_;
	print $D "%%%BEGIN_SECTION $sec%%%\n";
}

sub end_section($) {
	my $fd = shift;
	print $fd "\n%%%END_SECTION%%%\n\n";
}

sub print_registry($) {
	my $fd = shift;
	open(IN,"/etc/.snapvault") or die "Failed to open /etc/.snapvault for reading: $!. Is OSSV installed on this system?";
	my $inline = <IN>;
	close IN;
	print $fd $inline;
	chomp $inline;
	return $inline;
}

sub copy_all_changelog_files($) {
my $CLout = shift;
my $cldir = $installdir."/logs/";
mkdir $ CLout or opendir(DIR,$CLout) 
	or warn "Failed to create $CLout dir $! \n";closedir DIR;
mydircopy($cldir,$CLout) or print "Failed to copy all Changelog files $!\n";
}

my $NoFtlPth = 0;
my $CPRFComp = 0;
my $KeepMode = 0;
my $MaxDepth = 0;
my $CopyLink = 0;
my $BdTrgWrn = 0;
my $SkipFlop = 0;
my $CondCopy = {};
my $ok_todo_asper_condcopy = sub {
    my $org = shift;
    my $copy = 1;
    if(exists $CondCopy->{$org}) {
        if($CondCopy->{$org}{'md5'}) {

        }
        if($copy) {
        }
    }
    return $copy;
};
sub mydircopy {
   my $globstar = 0;
   my $_zero = $_[0];
   my $_one = $_[1];
   if ( substr( $_zero, ( 1 * -1 ), 1 ) eq '*') {
       $globstar = 1;
       $_zero = substr( $_zero, 0, ( length( $_zero ) - 1 ) );
   }
   warn "$_zero and $_[1] are the same" if $_zero eq $_[1]; 
   warn "$_zero is not a directory" if !-d $_zero;
   warn "$_[1] is not a directory" if -e $_[1] && !-d $_[1];

   if(!-d $_[1]) {
      pathmk($_[1], $NoFtlPth) or return;
   } else {
      if($CPRFComp && !$globstar) {
         my @parts = File::Spec->splitdir($_zero);
         while($parts[ $#parts ] eq '') { pop @parts; }
         $_one = File::Spec->catdir($_[1], $parts[$#parts]);
      }
   }
   my $baseend = $_one;
   my $level   = 0;
   my $filen   = 0;
   my $dirn    = 0;

   my $recurs; #must be my()ed before sub {} since it calls itself
   $recurs =  sub {
      my ($str,$end,$buf) = @_;
      $filen++ if $end eq $baseend; 
      $dirn++ if $end eq $baseend;
      mkdir $end or return if !-d $end;
      chmod scalar((stat($str))[2]), $end if $KeepMode;
      if($MaxDepth && $MaxDepth =~ m/^\d+$/ && $level >= $MaxDepth) {
         return ($filen,$dirn,$level) if wantarray;
         return $filen;
      }
      $level++;

      opendir(STR_DH, $str) or return;
      my @files = grep( $_ ne '.' && $_ ne '..', readdir(STR_DH));
      closedir STR_DH;

      for my $file (@files) {
          my ($file_ut) = $file =~ m{ (.*) }xms;
          my $org = File::Spec->catfile($str, $file_ut);
          my $new = File::Spec->catfile($end, $file_ut);
          if(-l $org && ($CopyLink || !-e readlink($_[0])) ) {
              die "Copying a symlink ($_[0]) whose target does not exist" 
                  if !-e readlink($_[0]) && $BdTrgWrn;
              symlink readlink($org), $new or return;
          } 
          elsif(-d $org) {
              $recurs->($org,$new,$buf) if defined $buf;
              $recurs->($org,$new) if !defined $buf;
              $filen++;
              $dirn++;
          } 
          else {
              #if($ok_todo_asper_condcopy->($org)) {
                  if($SkipFlop) {
                      copy($org,$new) or next if !defined $buf;                      
                  }
                  else {
                      copy($org,$new) or return if !defined $buf;
                  }
                  chmod scalar((stat($org))[2]), $new if $KeepMode;
                  $filen++;
              #}
          }
      }
      1;
   };

   $recurs->($_zero, $_one, $_[2]) or return;
   return wantarray ? ($filen,$dirn,$level) : $filen;
}


sub copy_all_trace_files($) {
my $traceout = shift;
my $tracedir = $installdir."/trace";
mkdir "$traceout/trace" or opendir(DIR,"$traceout/trace") or 
	warn "Failed to create outdir $traceout/trace error $! \n"; 
closedir DIR;

opendir(DIR,$tracedir) or warn "Can't open $tracedir: $!";
while( defined (my $file = readdir DIR) ) {
    copy("$tracedir/$file","$traceout/trace/$file") or 
		warn "Cannot copy file $file $! \n" 
	if (!(-d $file) and index($file,"qsm")eq 0);
	}
closedir(DIR);
}

sub print_hostname($) {
	my $fd = shift;
	print $fd `hostname`;
}

sub print_os($) {
	my $fd = shift;
	print $fd `uname -a`;
	if ( $esxBinary ) {
		print $fd `cat /etc/vmware-release 2>&1`;
		print $fd `"$installdir/uninstall/uname" -a`
	}
}

sub check_esx_binary {
	if ( -f "$installdir/uninstall/uname" ) {
		return 1;
	} else {
		return 0;
	}
}

sub print_memory($) {
	my $fd = shift;
	if($^O eq "irix")  {
		print $fd `sar -R 1`;
		print $fd "(All numbers refer to memory pages)\n";
	} else {
		print $fd `vmstat 4 2`;
	}
}

sub print_volume($) {
	my $fd = shift;
	print $fd `df -k`;
}

sub print_procs($) {
	my $fd = shift;
	print $fd `ps -eaf`;
}

sub print_snapvault_cmd($$$) {
	my ($fd, $installdir, $cmd) = @_;
	print $fd `$installdir/bin/snapvault $cmd`;
}

sub print_snapvault_status($$) {
	print_snapvault_cmd($_[0], $_[1], "status");
}

sub print_snapvault_destinations($$) {
	print_snapvault_cmd($_[0], $_[1], "destinations");
}

sub print_snapvault_status_l($$) {
	print_snapvault_cmd($_[0], $_[1], "status -l");
}

sub print_file($$) {
	my ($fd, $file) = @_;
	chomp($file);
	unless(open(KD, $file)) {
		print $fd "Failed to open $file for reading: $!\n";
		return undef;
	}
	print $fd "\n$file\n";
	print $fd <KD>;
	close KD;
	return 1;
}

sub print_releasedef($$) {
	my ($fd, $installdir) = @_;
	print_file($fd, "$installdir/RELEASEDEF");
}

sub print_snapvault($$) {
	my (@logfilelist, $logfile);
	my ($fd, $installdir) = @_;
	print_file($fd, "$installdir/etc/snapvault");
	use File::Glob qw(:globally :case);
	@logfilelist = <$installdir/etc/snapvault.{[0-9]*}>;
	foreach $logfile (@logfilelist) {
		print_file($fd, "$logfile");
	}
}

sub print_configure_cfg($$) {
	print_file($_[0], "$_[1]/config/configure.cfg");
}

sub print_path_exclude($$) {
	print_file($_[0], "$_[1]/etc/path-exclude.txt");
}

sub print_file_exclude($$) {
	print_file($_[0], "$_[1]/etc/file-exclude.txt");
}

sub print_db_dir($$) {
	my($fd, $installdir) = @_;
	print $fd `ls -lR $installdir/db`;
}

sub print_svinstallcheck($$) {
	my($fd, $installdir) = @_;
	print $fd `$installdir/bin/svinstallcheck`;
}

sub print_rcmd($$$$) {
	my($fd, $secondary, $username, $cmd) = @_;
	my $pwd = $username ? "-l $username" : "";
	print $fd `rsh $secondary $pwd $cmd`;
}

sub print_esx_volumes($) {
	my $fd = shift;
	print $fd `ls -al /vmfs/volumes 2>&1`;
}

sub print_vmware_cfg($$) {
	print_file($_[0], "$_[1]/config/ossv_vmware.cfg");
}

sub copy_vmware_vmxfiles($$) {
	my ($fd, $outputdir) = @_;
	my $vmxFilesDir = "$outputdir/vmxFiles";
	mkdir $vmxFilesDir or opendir(DIR,$vmxFilesDir) or
		warn "Failed to create output folder $vmxFilesDir $! \n";closedir DIR;

	unless ( -d "/vmfs/volumes" ) {
		print $fd "The directory /vmfs/volumes was not found";
		return;
	}

	my @files=`find /vmfs/volumes -name '*.vmx' 2>&1`;
	print $fd @files;
	print $fd "\n";

	my $fileNum = 0;
	for my $file (@files) {
		chomp $file;
		my $filename = `basename "$file"`;
		chomp $filename;
		copy($file,"$vmxFilesDir/$fileNum.$filename");
		print $fd "Copied $file to $vmxFilesDir/$fileNum.$filename\n";
		$fileNum++;
	}
}

sub get_vmware_vminfo($$) {
	my ($fd, $installdir)  = @_;
	my $username = `$installdir/bin/svpassword -g username`;
	my $password = `$installdir/bin/svpassword -g password`;
	my $vchost   = `$installdir/bin/svpassword -g vchost`;

	chomp $username;
	chomp $password;
	chomp $vchost;

	if ( 0 == length ($username)) {
		print $fd "An empty vmware username was returned";
		$username="\"\"";
	}
	if ( 0 == length ($password)) {
		print $fd "An empty vmware password was returned.\n";
		$password="\"\"";
	}
	if ( 0 == length ($vchost)) {
		print $fd "An empty vmware vchost was returned.\n";
		$vchost="localhost";
	}

	my $vminfo = `/usr/sbin/vcbVmName -h $vchost -u $username -p $password -s Any 2>&1`;
	print $fd $vminfo;
}

